@@ -1,27 +1,44 @@
-import warnings
-from typing import Any, Callable, cast, Dict, List, Optional, TYPE_CHECKING, Union
+"""
+GitLab API:
+https://docs.gitlab.com/ee/api/projects.html
+"""
+from typing import (
+    Any,
+    Callable,
+    cast,
+    Dict,
+    Iterator,
+    List,
+    Optional,
+    TYPE_CHECKING,
+    Union,
+)
 
 import requests
 
 from gitlab import cli, client
 from gitlab import exceptions as exc
 from gitlab import types, utils
-from gitlab.base import RequiredOptional, RESTManager, RESTObject
+from gitlab.base import RESTManager, RESTObject
 from gitlab.mixins import (
     CreateMixin,
     CRUDMixin,
+    GetWithoutIdMixin,
     ListMixin,
     ObjectDeleteMixin,
     RefreshMixin,
     SaveMixin,
     UpdateMixin,
 )
+from gitlab.types import RequiredOptional
 
 from .access_requests import ProjectAccessRequestManager  # noqa: F401
+from .artifacts import ProjectArtifactManager  # noqa: F401
 from .audit_events import ProjectAuditEventManager  # noqa: F401
 from .badges import ProjectBadgeManager  # noqa: F401
 from .boards import ProjectBoardManager  # noqa: F401
 from .branches import ProjectBranchManager, ProjectProtectedBranchManager  # noqa: F401
+from .ci_lint import ProjectCiLintManager  # noqa: F401
 from .clusters import ProjectClusterManager  # noqa: F401
 from .commits import ProjectCommitManager  # noqa: F401
 from .container_registry import ProjectRegistryRepositoryManager  # noqa: F401
@@ -29,12 +46,18 @@ from .custom_attributes import ProjectCustomAttributeManager  # noqa: F401
 from .deploy_keys import ProjectKeyManager  # noqa: F401
 from .deploy_tokens import ProjectDeployTokenManager  # noqa: F401
 from .deployments import ProjectDeploymentManager  # noqa: F401
-from .environments import ProjectEnvironmentManager  # noqa: F401
+from .environments import (  # noqa: F401
+    ProjectEnvironmentManager,
+    ProjectProtectedEnvironmentManager,
+)
 from .events import ProjectEventManager  # noqa: F401
 from .export_import import ProjectExportManager, ProjectImportManager  # noqa: F401
 from .files import ProjectFileManager  # noqa: F401
 from .hooks import ProjectHookManager  # noqa: F401
+from .integrations import ProjectIntegrationManager, ProjectServiceManager  # noqa: F401
+from .invitations import ProjectInvitationManager  # noqa: F401
 from .issues import ProjectIssueManager  # noqa: F401
+from .iterations import GroupIterationManager  # noqa: F401
 from .jobs import ProjectJobManager  # noqa: F401
 from .labels import ProjectLabelManager  # noqa: F401
 from .members import ProjectMemberAllManager, ProjectMemberManager  # noqa: F401
@@ -59,7 +82,6 @@ from .push_rules import ProjectPushRulesManager  # noqa: F401
 from .releases import ProjectReleaseManager  # noqa: F401
 from .repositories import RepositoryMixin
 from .runners import ProjectRunnerManager  # noqa: F401
-from .services import ProjectServiceManager  # noqa: F401
 from .snippets import ProjectSnippetManager  # noqa: F401
 from .statistics import (  # noqa: F401
     ProjectAdditionalStatisticsManager,
@@ -80,6 +102,10 @@ __all__ = [
     "ProjectForkManager",
     "ProjectRemoteMirror",
     "ProjectRemoteMirrorManager",
+    "ProjectStorage",
+    "ProjectStorageManager",
+    "SharedProject",
+    "SharedProjectManager",
 ]
 
 
@@ -125,21 +151,23 @@ class ProjectGroupManager(ListMixin, RESTManager):
         "shared_min_access_level",
         "shared_visible_only",
     )
-    _types = {"skip_groups": types.CommaSeparatedListAttribute}
+    _types = {"skip_groups": types.ArrayAttribute}
 
 
 class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTObject):
-    _short_print_attr = "path"
+    _repr_attr = "path_with_namespace"
 
     access_tokens: ProjectAccessTokenManager
     accessrequests: ProjectAccessRequestManager
     additionalstatistics: ProjectAdditionalStatisticsManager
     approvalrules: ProjectApprovalRuleManager
     approvals: ProjectApprovalManager
+    artifacts: ProjectArtifactManager
     audit_events: ProjectAuditEventManager
     badges: ProjectBadgeManager
     boards: ProjectBoardManager
     branches: ProjectBranchManager
+    ci_lint: ProjectCiLintManager
     clusters: ProjectClusterManager
     commits: ProjectCommitManager
     customattributes: ProjectCustomAttributeManager
@@ -154,8 +182,11 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
     groups: ProjectGroupManager
     hooks: ProjectHookManager
     imports: ProjectImportManager
+    integrations: ProjectIntegrationManager
+    invitations: ProjectInvitationManager
     issues: ProjectIssueManager
     issues_statistics: ProjectIssuesStatisticsManager
+    iterations: GroupIterationManager
     jobs: ProjectJobManager
     keys: ProjectKeyManager
     labels: ProjectLabelManager
@@ -170,6 +201,7 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
     pagesdomains: ProjectPagesDomainManager
     pipelines: ProjectPipelineManager
     pipelineschedules: ProjectPipelineScheduleManager
+    protected_environments: ProjectProtectedEnvironmentManager
     protectedbranches: ProjectProtectedBranchManager
     protectedtags: ProjectProtectedTagManager
     pushrules: ProjectPushRulesManager
@@ -179,6 +211,7 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
     runners: ProjectRunnerManager
     services: ProjectServiceManager
     snippets: ProjectSnippetManager
+    storage: "ProjectStorageManager"
     tags: ProjectTagManager
     triggers: ProjectTriggerManager
     users: ProjectUserManager
@@ -444,22 +477,39 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
 
         url = f"/projects/{self.encoded_id}/uploads"
         file_info = {"file": (filename, filedata)}
-        data = self.manager.gitlab.http_post(url, files=file_info)
+        data = self.manager.gitlab.http_post(url, files=file_info, **kwargs)
 
         if TYPE_CHECKING:
             assert isinstance(data, dict)
         return {"alt": data["alt"], "url": data["url"], "markdown": data["markdown"]}
 
+    @cli.register_custom_action("Project")
+    @exc.on_http_error(exc.GitlabRestoreError)
+    def restore(self, **kwargs: Any) -> None:
+        """Restore a project marked for deletion.
+
+        Args:
+            **kwargs: Extra options to send to the server (e.g. sudo)
+
+        Raises:
+            GitlabAuthenticationError: If authentication is not correct
+            GitlabRestoreError: If the server failed to perform the request
+        """
+        path = f"/projects/{self.encoded_id}/restore"
+        self.manager.gitlab.http_post(path, **kwargs)
+
     @cli.register_custom_action("Project", optional=("wiki",))
     @exc.on_http_error(exc.GitlabGetError)
     def snapshot(
         self,
         wiki: bool = False,
         streamed: bool = False,
-        action: Optional[Callable] = None,
+        action: Optional[Callable[[bytes], None]] = None,
         chunk_size: int = 1024,
+        *,
+        iterator: bool = False,
         **kwargs: Any,
-    ) -> Optional[bytes]:
+    ) -> Optional[Union[bytes, Iterator[Any]]]:
         """Return a snapshot of the repository.
 
         Args:
@@ -467,6 +517,8 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
             streamed: If True the data will be processed by chunks of
                 `chunk_size` and each chunk is passed to `action` for
                 treatment.
+            iterator: If True directly return the underlying response
+                iterator
             action: Callable responsible of dealing with chunk of
                 data
             chunk_size: Size of each chunk
@@ -481,11 +533,13 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
         """
         path = f"/projects/{self.encoded_id}/snapshot"
         result = self.manager.gitlab.http_get(
-            path, streamed=streamed, raw=True, **kwargs
+            path, streamed=streamed, raw=True, wiki=wiki, **kwargs
         )
         if TYPE_CHECKING:
             assert isinstance(result, requests.Response)
-        return utils.response_content(result, streamed, action, chunk_size)
+        return utils.response_content(
+            result, streamed, action, chunk_size, iterator=iterator
+        )
 
     @cli.register_custom_action("Project", ("scope", "search"))
     @exc.on_http_error(exc.GitlabSearchError)
@@ -546,101 +600,34 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
 
     @cli.register_custom_action("Project", ("to_namespace",))
     def transfer_project(self, *args: Any, **kwargs: Any) -> None:
-        warnings.warn(
-            "The project.transfer_project() method is deprecated and will be "
-            "removed in a future version. Use project.transfer() instead.",
-            DeprecationWarning,
+        utils.warn(
+            message=(
+                "The project.transfer_project() method is deprecated and will be "
+                "removed in a future version. Use project.transfer() instead."
+            ),
+            category=DeprecationWarning,
         )
         return self.transfer(*args, **kwargs)
 
-    @cli.register_custom_action("Project", ("ref_name", "job"), ("job_token",))
-    @exc.on_http_error(exc.GitlabGetError)
-    def artifacts(
-        self,
-        ref_name: str,
-        job: str,
-        streamed: bool = False,
-        action: Optional[Callable] = None,
-        chunk_size: int = 1024,
-        **kwargs: Any,
-    ) -> Optional[bytes]:
-        """Get the job artifacts archive from a specific tag or branch.
-
-        Args:
-            ref_name: Branch or tag name in repository. HEAD or SHA references
-            are not supported.
-            artifact_path: Path to a file inside the artifacts archive.
-            job: The name of the job.
-            job_token: Job token for multi-project pipeline triggers.
-            streamed: If True the data will be processed by chunks of
-                `chunk_size` and each chunk is passed to `action` for
-                treatment
-            action: Callable responsible of dealing with chunk of
-                data
-            chunk_size: Size of each chunk
-            **kwargs: Extra options to send to the server (e.g. sudo)
-
-        Raises:
-            GitlabAuthenticationError: If authentication is not correct
-            GitlabGetError: If the artifacts could not be retrieved
-
-        Returns:
-            The artifacts if `streamed` is False, None otherwise.
-        """
-        path = f"/projects/{self.encoded_id}/jobs/artifacts/{ref_name}/download"
-        result = self.manager.gitlab.http_get(
-            path, job=job, streamed=streamed, raw=True, **kwargs
-        )
-        if TYPE_CHECKING:
-            assert isinstance(result, requests.Response)
-        return utils.response_content(result, streamed, action, chunk_size)
-
     @cli.register_custom_action("Project", ("ref_name", "artifact_path", "job"))
     @exc.on_http_error(exc.GitlabGetError)
     def artifact(
         self,
-        ref_name: str,
-        artifact_path: str,
-        job: str,
-        streamed: bool = False,
-        action: Optional[Callable] = None,
-        chunk_size: int = 1024,
+        *args: Any,
         **kwargs: Any,
     ) -> Optional[bytes]:
-        """Download a single artifact file from a specific tag or branch from
-        within the job’s artifacts archive.
-
-        Args:
-            ref_name: Branch or tag name in repository. HEAD or SHA references
-                are not supported.
-            artifact_path: Path to a file inside the artifacts archive.
-            job: The name of the job.
-            streamed: If True the data will be processed by chunks of
-                `chunk_size` and each chunk is passed to `action` for
-                treatment
-            action: Callable responsible of dealing with chunk of
-                data
-            chunk_size: Size of each chunk
-            **kwargs: Extra options to send to the server (e.g. sudo)
-
-        Raises:
-            GitlabAuthenticationError: If authentication is not correct
-            GitlabGetError: If the artifacts could not be retrieved
-
-        Returns:
-            The artifacts if `streamed` is False, None otherwise.
-        """
-
-        path = (
-            f"/projects/{self.encoded_id}/jobs/artifacts/{ref_name}/raw/"
-            f"{artifact_path}?job={job}"
-        )
-        result = self.manager.gitlab.http_get(
-            path, streamed=streamed, raw=True, **kwargs
+        utils.warn(
+            message=(
+                "The project.artifact() method is deprecated and will be "
+                "removed in a future version. Use project.artifacts.raw() instead."
+            ),
+            category=DeprecationWarning,
         )
+        data = self.artifacts.raw(*args, **kwargs)
         if TYPE_CHECKING:
-            assert isinstance(result, requests.Response)
-        return utils.response_content(result, streamed, action, chunk_size)
+            assert data is not None
+            assert isinstance(data, bytes)
+        return data
 
 
 class ProjectManager(CRUDMixin, RESTManager):
@@ -680,6 +667,7 @@ class ProjectManager(CRUDMixin, RESTManager):
             "jobs_enabled",
             "lfs_enabled",
             "merge_method",
+            "merge_pipelines_enabled",
             "merge_requests_access_level",
             "merge_requests_enabled",
             "mirror_trigger_builds",
@@ -704,6 +692,7 @@ class ProjectManager(CRUDMixin, RESTManager):
             "snippets_enabled",
             "squash_option",
             "tag_list",
+            "topics",
             "template_name",
             "template_project_id",
             "use_custom_template",
@@ -744,6 +733,7 @@ class ProjectManager(CRUDMixin, RESTManager):
             "jobs_enabled",
             "lfs_enabled",
             "merge_method",
+            "merge_pipelines_enabled",
             "merge_requests_access_level",
             "merge_requests_enabled",
             "mirror_overwrites_diverged_branches",
@@ -774,6 +764,7 @@ class ProjectManager(CRUDMixin, RESTManager):
             "squash_option",
             "suggestion_commit_message",
             "tag_list",
+            "topics",
             "visibility",
             "wiki_access_level",
             "wiki_enabled",
@@ -810,6 +801,7 @@ class ProjectManager(CRUDMixin, RESTManager):
     _types = {
         "avatar": types.ImageAttribute,
         "topic": types.CommaSeparatedListAttribute,
+        "topics": types.ArrayAttribute,
     }
 
     def get(self, id: Union[str, int], lazy: bool = False, **kwargs: Any) -> Project:
@@ -1083,3 +1075,39 @@ class ProjectRemoteMirrorManager(ListMixin, CreateMixin, UpdateMixin, RESTManage
         required=("url",), optional=("enabled", "only_protected_branches")
     )
     _update_attrs = RequiredOptional(optional=("enabled", "only_protected_branches"))
+
+
+class ProjectStorage(RefreshMixin, RESTObject):
+    pass
+
+
+class ProjectStorageManager(GetWithoutIdMixin, RESTManager):
+    _path = "/projects/{project_id}/storage"
+    _obj_cls = ProjectStorage
+    _from_parent_attrs = {"project_id": "id"}
+
+    def get(self, **kwargs: Any) -> ProjectStorage:
+        return cast(ProjectStorage, super().get(**kwargs))
+
+
+class SharedProject(RESTObject):
+    pass
+
+
+class SharedProjectManager(ListMixin, RESTManager):
+    _path = "/groups/{group_id}/projects/shared"
+    _obj_cls = SharedProject
+    _from_parent_attrs = {"group_id": "id"}
+    _list_filters = (
+        "archived",
+        "visibility",
+        "order_by",
+        "sort",
+        "search",
+        "simple",
+        "starred",
+        "with_issues_enabled",
+        "with_merge_requests_enabled",
+        "min_access_level",
+        "with_custom_attributes",
+    )
